; Programm
; SPIELE-EPROM
; Autor: Gunther Zielosko, reass. und neu zusammengesetzt Volker Pohlers, 2003
;------------------------------------------------------------------------------
; ich hatte einen abgeschriebenen DUMP des ROMs bis Adresse 116F und ein paar
; weitere Notizen. Die Spiele 'Spielhlle' und 'fallende Leuchtbalken' (aus Heft 3)
; stimmen mit neuem ORG zumindest im Speicherbereich berein. Die Spiele aus Heft 1
; 'Reaktionstest' und 'Wrfel' muten fr eine ROM-Version leicht gendert werden.
;
; 31.08.03 in Reakttest ist noch ein Fehler :-(
;------------------------------------------------------------------------------
; Start auf Adr. 1000 (Zusatz-ROM 3)
; ber die Tasten 1..5 sind 5 verschiedene Spiele startbar.
;
; 1000-1030	Men				100% original ROM
; 1030-116f	Mastermind			100% original ROM (mehr vom Original-ROM habe ich leider nicht mehr)
; 1170-1245	Reakttest			meine Umsetzung ist viel krzer als hier angegeben ...
; 1250-12e0	elektron. Wrfel		Ende falsch aufgeschrieben ??? ich habe 4 Byte mehr ...
; 12f0-13f1	Spielhlle			100% Buch
; 1400-14e9	fallende Leuchtbalken		100% Buch
;
;------------------------------------------------------------------------------

		CPU Z80
		include	lc80

;------------------------------------------------------------------------------
; Men
; ber die Tasten 1..5 sind 5 verschiedene Spiele startbar.
; Taste 6 ist vorbereitet, aber noch unbelegt.
;------------------------------------------------------------------------------
		org 1000h

anf:		ld	ix, txt_spiel
		call	dak1
		cp	1
		jp	z,logik			; Mastermind
		cp	2
		jp	z,reakttest		; Reaktionstest
		cp	3
		jp	z,wuerfel		; elektronischer Wrfel
		cp	4
		jp	z,bandit		; Spielhlle
		cp	5
		jp	z,taler			; fallende Leuchtbalken
		cp	6
		jp	z,0ffffh
		jp	anf
;
txt_spiel:	db	05dh,0c2h,0ceh,021h,04fh,0aeh	; "Spiel?"
;
;------------------------------------------------------------------------------
; Mastermind
; 0..7, auch doppelt
; '+' - Start
; Eingabe Versuch -> Anzeige Versuch|Nr
; '+' Anzeige links Position, rechts Farbe (z.B. 0P|1F - 1 richtig, aber falsche Stelle)
; bei 4Ps -> Fanfare
; bel. Taste -> nchster Versuch
;------------------------------------------------------------------------------
		
		org	1030h
logik:
; Speicherbereiche lschen
		ld	b,01fh
		ld	hl,2300h
		ld	a,0
logik_m1:	ld	(hl),a
		inc	hl
		djnz	logik_m1
; 2 Zufallsziffern eintragen
		ld	hl,2308h
		call	logik_zufaz
; Laufschrift LOGICSPIEL
logik_m2:	ld	c,0fh
		ld	ix,txt_logic
logik_m3:	ld	b,10h
logik_m4:	call	dak2
		cp	0ah			; Taste '+'
		jr	z,logik_m5
		djnz	logik_m4		
		dec	ix
		dec	c
		jr	nz,logik_m3
		jr	logik_m2
; 2 Zufallszahlen eintragen, Versuchsnr. auf 0 setzen
logik_m5:	ld	hl,230ah
		call	logik_zufaz
		ld	a,1
		ld	(230Ch),a
; Darstellen des Speicherbereiches Versuch und Versuchsnummer
logik_m6:	ld	hl,2312h
		ld	ix,2306h
		call	logik_anz
		call	dak1
		cp	10h			; Taste '+' ?
		jr	z,logik_m7
		call	logik_verpro
		jr	logik_m6
; aktuelle Versuchsnummer aus 230ch holen, um 1 erhhen, Ergebnis BCD-
; korrigieren, untere 4 Bit auf 230ch, obere 4 Bit auf 230dh ablegen
logik_m7:	call	logik_vergl
		ld	a,(230ch)
		add	a,1
		daa
		ld	(230ch),a
		and	0f0h
		rrca
		rrca
		rrca
		rrca
		ld	(230dh),a
; "P" in 2316h eintragen
		ld	a,04fh
		ld	(2316h),a
; "F" in 2312h eintragen
		ld	a,04eh
		ld	(2312h),a
; F-Zahl in 7-Segment-Form in 2313h laden
		ld	a,(2306h)
		call	oneseg
		ld	(2313h),a
; P-Zahl in 7-Segment-Form in 2317h laden
		ld	a,(2307h)
		call	oneseg
		ld	(2317h),a
		cp	2bh			; Vergl. mit '4'
		jr	nz,logik_m8
logik_m9:	ld	ix,2312h
		call	dak1
		jp	logik_m6
;
logik_m8:	ld	iy,logik_noten
		call	music
		jr	logik_m9
;
; Text LOGICSPIEL als Laufschrift
		db	0,0
		db	000h,000h,0c2h,0ceh,021h,04fh	; 'PIEL  '
		db	0aeh,0c6h,021h,0e6h,0e7h,0c2h	; 'LOGICS'
txt_logic:	db	0,0,0,0,0,0
;		
; A mit 2 Hexa-Ziffern aus dem Refresh-Register laden, Bit 3 und 7 ausblenden
; und so umformen, da je eine Ziffer (0..7) in HL und HL+1 abgelegt wird.
logik_zufaz:	ld	a,R
		and	77h
		rld
		rrca
		rrca
		rrca
		rrca
		inc	hl
		rld
		ret
;
; 6maliges Umformen der Tipzahlen und der Versuchsnummer in 7-Segment-Form,
; Vorbereiten der Anzeige. Setzten des Punktes in Digit 4
logik_anz:	ld	b,6
logik_m10:	dec	hl
		dec	ix
		ld	a,(hl)
		call	oneseg
		ld	(ix+0),a
		djnz	logik_m10
		ld	hl,2302h
		set	4,(hl)
		ret
;
; Verschieben von 4 Tippziffern um eine Stelle
; letzte Ziffer in T4, T4->T3, T3->T2, T2-T1, T1 geht verloren
logik_verpro:	ld	b,4
		ld	hl,230dh
logik_m11:	inc	hl
		and	7
		rrd
		rr	(hl)
		rr	(hl)
		rr	(hl)
		rr	(hl)
		djnz	logik_m11
		ret
;
logik_vergl:	call	SOU1K			; Piepton
		ld	a,0
		ld	(2306h),a		; P und F lschen
		ld	(2307h),a
; Positionsvergleich 
		ld	b,4			
		ld	de,230dh
		ld	hl,2307h
logik_m12:	inc	de
		inc	hl			; (Z4)
		ld	a,(de)			; (T4)
		cp	(hl)			; T4 mit Z4 vergleichen
		jr	nz,logik_m13
;
		ld	a,(2307h)		; P um 1 erhhen
		inc	a
		ld	(2307h),a
logik_m13:	djnz	logik_m12			; nchstes Z und T
; Doppel der Zufallszahl Z1 bis Z4 auf 231ah bis 231dh ablegen (Z1' bis Z4')
		ld	hl,2308h
		ld	de,231ah
		ld	bc,4
		ldir
; Farbvergleich
		ld	b,4
		ld	hl,230eh
logik_m14:	ld	de,231ah
		ld	c,4
logik_m15:	ld	a,(de)
		cp	(hl)			; T4 mit Z4' vergl.
		jr	nz,logik_m16
;
		ld	a,(2306h)		; F um 1 erhhen
		inc	a
		ld	(2306h),a
		ld	a,0ffh			; Z4' mit 0FFh berschreiben
		ld	(de),a
		jr	logik_m17
;
logik_m16:	inc	de			; T4 mit Z3' vergl. usw.
		dec	c
		jr	nz,logik_m15
;
logik_m17:	inc	hl			; alles mit T3 wiederholen usw.
		djnz	logik_m14
;
		ld	hl,2307h		; P in (HL)
		ld	a,(2306h)		; Fneu = F-P in 2306h laden
		sub	(HL)
		ld	(2306h),a
		ret
;
logik_noten:	db	0bh,04h
		db	0fh,04h
		db	12h,04h
		db	17h,10h
		db	80h		



;------------------------------------------------------------------------------
; Spiel2: Reaktionstest
;------------------------------------------------------------------------------

		org	1170h
reakttest:
reakt_anf:	ld	hl,0
		ld	(reakt_treffer-1),hl
		ld	(reakt_anzeige),hl
		ld	(reakt_anzeige+2),hl
		ld	(reakt_anzeige+4),hl
		ld	b,0ffh
		ld	ix,reakt_anzeige
reakt_m1:	call	dak2
		djnz	reakt_m1
		ld	c,0ffh
reakt_m2:	ld	a,r
		and	0fh
		ld	(reakt_zufzahl),a
		call	oneseg
		ld	(reakt_anzeige),a
		ld	b,c
reakt_m3:	ld	a,33h
		ld	hl,reakt_zufzahl
		call	dak2
		add	a,lo(reakt_keys)
		ld	e,a
		ld	d,hi(reakt_keys)
		ld	a,(de)			; von dieser Adresse die Tastenwertigkeit holen
		cp	(hl)
		jr	z,reakt_m6
		djnz	reakt_m3
reakt_m4:	ld	a,c
		sub	2
		jr	c,reakt_m5
		ld	c,a
		jr	reakt_m2
reakt_m5:	jp	reakt_m8
;
reakt_m6:	push	bc
		ld	c,25h
		ld	hl,20h
		call	sound
		ld	a,(reakt_treffer)
		inc	a
		daa
		ld	(reakt_treffer),a
		ld	hl,reakt_anzeige+5
		call	twoseg
		pop	bc
reakt_m7:	call	dak2
		djnz	reakt_m7
		jp	reakt_m4
;
reakt_m8:	ld	a,(reakt_treffer)		
		ld	b,a
		ld	a,(reakt_rekord)
		cp	b
		jr	c,reakt_m10
		ld	hl,reakt_anzeige
		call	twoseg
		ld	c,1
reakt_m9:	ld	hl,2
		call	sound
		inc	c
		jr	nz,reakt_m9
		jp	reakt_m11
;
reakt_m10:	ld	a,(reakt_treffer)
		ld	(reakt_rekord),a
		ld	hl,reakt_anzeige
		call	twoseg
		push	ix
		ld	iy,reakt_fanfare
		call	music
		pop	ix
reakt_m11:	call	dak1
		jp	reakt_anf

reakt_keys:	db	0,0,0,0			; EX,ST,LD,(leer)
		db	0,1,2,3			; 0,1,2,3
		db	4,5,0,7			; 4,5,+,7
		db	8,9,0eh,0bh		; 8,9,E,B
		db	0ch,0dh,0ah,0fh		; C,D,A,F
		db	0ffh,0ffh,6,0ffh	; ADR,DAT,6,-
reakt_fanfare:	db	0bh,04h
		db	0fh,04h
		db	12h,04h
		db	17h,10h
		db	80h

		org	2100h

reakt_treffer	equ	2100h
reakt_zufzahl:	equ	2101h
reakt_anzeige:	equ	2102h
reakt_rekord:	equ	2108h

;------------------------------------------------------------------------------
; Spiel3: 
; s. "Hinweise zur Anwendung des Lerncomputers LC 80", Teil 1, Seite 41-45
; 1250-12e0	elektron. Wrfel
;------------------------------------------------------------------------------

		org	1250h

wuerfel:	ld	hl,wuerfel_text0
		ld	de,wuerfel_text
		ld	bc,12
		ldir

wuerfel_start:	ld	ix, wuerfel_text
wuerfel_start1:	call	DAK1			; Anzeigen von 2100...2105
		cp	11h			; "-" - Taste?
		jr	nz, wuerfel_start1
		ld	ix, wuerfel_text+6
;
wuerfel_zahl1:	call	DAK2			; Anzeigen von 2106...210B
		ld	c, 21h			; "1" in 7-Segment-Darstellung
		cp	0ah			; "+" - Taste?
		jp	z, wuerfel_wahl
		jp	wuerfel_zahl2
wuerfel_zahl2:	call	DAK2
		ld	c, 0CDh			; "2"
		cp	0ah			; "+" - Taste?
		jp	z, wuerfel_wahl
		jp	wuerfel_zahl3
wuerfel_zahl3:	call	DAK2
		ld	c, 0ADh			; "3"
		cp	0ah			; "+" - Taste?
		jp	z, wuerfel_wahl
		jp	wuerfel_zahl4
wuerfel_zahl4:	call	DAK2
		ld	c, 2Bh			; "4"
		cp	0ah			; "+" - Taste?
		jp	z, wuerfel_wahl
		jp	wuerfel_zahl5
wuerfel_zahl5:	call	DAK2
		ld	c, 0AEh			; "5"
		cp	0ah			; "+" - Taste?
		jp	z, wuerfel_wahl
		jp	wuerfel_zahl6
wuerfel_zahl6:	call	DAK2
		ld	c, 0EEh			; "6"
		cp	0ah			; "+" - Taste?
		jp	z, wuerfel_wahl
		jp	wuerfel_zahl1
;
wuerfel_wahl:	ld	a, c
		ld	(wuerfel_text+5), a
;		
		ld	c, 25h			; Ton
		ld	hl, 100h
		call	SOUND
;
		ld	ix, wuerfel_text+6
		ld	c, 6
wuerfel_anz1:	ld	b, 2			; Schnelligkeit der Laufschrift
wuerfel_anz2:	call	DAK2			; Laufschrift und Anzeige
		djnz	wuerfel_anz2
		dec	ix
		dec	c
		jr	nz, wuerfel_anz1
		jp	wuerfel_start


wuerfel_text0:	db 	0h
		db 	0h
		db 	0h
		db 	0h
		db 	0h
		db    0E7h			; Anzeige der "0" bei Beginn
		db    	0h
		db    	0h
		db    	0h
		db    	0h
		db    	0h
		db    	0h

wuerfel_text	equ	2100h		



;------------------------------------------------------------------------------
; Spiel4: Einarmiger Bandit
;------------------------------------------------------------------------------

		org	12f0h
bandit:
; 10,00 M in Bank laden
		ld	hl,1000h
		ld	(2216h),hl
; 03 in 2211 eintragen
bandit_m1:	ld	a,03
		ld	(2211h),a
; Speicherbereich von 2200 bis 220c lschen
		ld	b,0ch
		ld	hl,2200h
		ld	a,0
bandit_m2:	ld	(hl),a
		inc	hl
		djnz	bandit_m2
		ld	ix,2200h		; Anzeigebereich 1
bandit_m3:	call	bandit_rotanz
		cp	05h			; '1'?  1. Ziffer
		jr	nz,bandit_m3			; Stop?
		call	bandit_stop
bandit_m4:	call	bandit_rotanz
		cp	06h			; '2'?  2. Ziffer
		jr	nz,bandit_m4			; Stop?
		call	bandit_stop
bandit_m5:	call	bandit_rotanz
		cp	07h			; '3'?  3. Ziffer
		jr	nz,bandit_m5			; Stop?
		call	SOU1K			; original: SOU1K+1
		call	bandit_vergleich
; Wenn Gewinn=0, dann B mit Gewinnzahl (3 oder 9) laden und B-mal Wechselton erzeugen
		ld	a,(2206h)
		cp	0
		jr	z,bandit_m7
		ld	b,a
bandit_m6:	push	bc
		call	SOU1K			; original: SOU1K+1
		call	SOU2K
		pop	bc
		djnz	bandit_m6
bandit_m7:	call	bandit_bilanz
bandit_m8:	call	bandit_anz
		cp	04h			; '0'? Neues Spiel?
		jr	z,bandit_m1
		cp	0ah			; '+'? Anzeige des Bankguthabens?
		jr	nz,bandit_m8
; Eintragen des Bankguthabens in Anzeigebereich 2, Lschen der letzten Stellen,
; Setzen des Dezimalpunktes, Anzeige
		ld	de,(2216h)
		call	ADRSDP
		ld	hl,0
		ld	(DATLED),hl
		ld	hl,DATLED+4
		set	4,(hl)
		ld	ix,DATLED
bandit_m9:	call	dak1
		cp	0			; '0'? Neues Spiel?
		jr	nz,bandit_m9
		jr	bandit_m1
;
bandit_rotanz:		
; Erzeugen der 3 rollenden Ziffern (Anzahl der noch rollenden Ziffern in B)
		ld	b,(ix+11h)
		ld	hl,2209h
bandit_r1:	ld	a,(hl)
		inc	a
		cp	7			; Endzahl+1
		jr	nz,bandit_r2
		ld	a,1			; Anfangszahl
bandit_r2:	ld	(hl),a
		inc	hl
		djnz	bandit_r1
; bertragen der 3 rollenden Ziffern in den 7-Segment-Code und Ablage
; im Anzeigebereich 1
bandit_anz:	ld	b,3
		ld	hl,2209h
		ld	de,2203h
bandit_r3:	ld	a,(hl)
		call	oneseg
		ld	(de),a
		inc	hl
		inc	de
		djnz	bandit_r3
		call	dak2			; Anzeige
		ret
;
bandit_Stop:
; Herabsetzen der Anzahl der rotierenden Ziffern
		ld	a,(2211h)
		dec	a
		ld	(2211h),a
		call	SOU1K			; original: SOU1K+1
		ret
;
bandit_vergleich:
; Vergleich der 1. mit der 2. Ziffer
		ld	hl,220bh
		ld	a,(hl)
		dec	hl
		cp	(hl)
		jr	nz,bandit_v1
; Gewinn 3, wenn 1. und 2. Ziffer gleich, Umformen in 7-Segment-Code
		ld	a,3
		ld	(2206h),a
		call	oneseg
		ld	(2200h),a
; Vergleich der 2. mit der 3. Ziffer
		ld	a,(hl)
		dec	hl
		cp	(hl)
		jr	nz,bandit_v1
; Gewinn 9, wenn 1., 2. und 3. Ziffer gleich, Umformung
		ld	a,9
		ld	(2206h),a
		call	oneseg
		ld	(2200h),a
bandit_v1:	ret
;
bandit_bilanz:
; Gewinnziffer 4x links verschieben (aus 09 wird 90), Ergebnis in Register D laden
		ld	a,(2206h)
		rlca
		rlca
		rlca
		rlca
		and	a,0f0h
		ld	d,a
; Gewinn vom Guthaben (Groschen) abziehen, bei bertrag Markbetrag um 1 vermindern,
; neue Guthaben abspeichern
		ld	a,(2216h)
		sub	d
		daa
		ld	(2216h),a
		jr	nc,bandit_b1
		ld	a,(2217h)
		sub	1
		daa
		ld	(2217h),a
; Spieleinsatz (10) zu Guthaben (Groschen) addieren, bei bertrag Markbetrag um 1
; erhhen, Guthaben abspeichern
bandit_b1:	ld	a,(2216h)
		add	a,10h
		daa
		ld	(2216h),a
		jr	nc,bandit_b2
		ld	a,(2217h)
		add	a,1
		daa
		ld	(2217h),a
bandit_b2:	ret
;


;------------------------------------------------------------------------------
; Spiel5: Sterntaler (fallende Leuchtbalken)
;------------------------------------------------------------------------------

		org	1400h
taler:
; Lschen des Anzeige- und Speicherbereiches
taler_h0:	ld	b,10h
		ld	hl,2210h
taler_h1:	ld	(hl),0
		dec	hl
		djnz	taler_h1
		ld	ix,2200h
; Punkt setzen
		ld	(hl),10h		
		ld	a,l
		ld	(2220h),a		; Position des Punktes in 2220
		ld	a,1ch			; Startpunkt des Rckwrtsszhlers
		ld	(2222h),a
taler_h2:	ld	d,20h
taler_h3:	call	taler_sterne
; Schrittweise Erhhung der Geschwindigkeit (Stufengre in 2030 vernderbar,
; Startwert in 2015), Stufen 01, 02, 04
		ld	a,(2222h)
		ld	b,a
taler_h4:	push	bc
		call	taler_faenger
		pop	bc
		djnz	taler_h4
		dec	d
		jr	nz,taler_h3
		ld	a,(2222h)
		sub	02			; 04, 01
		jr	z,taler_h5
		ld	(2222h),a
		jr	taler_h2
taler_h5:	ld	de,(2208h)
; Punktestand im Anzeigebereich 2, Anzeige
		call	adrsdp
		ld	ix,DATLED
taler_h6:	call	dak1
		cp	10h			; '+'?
		jr	z,taler_h0
		jr	taler_h6
		
;		org	2050h
		ds	4
;
taler_faenger:
		ld	a,(2220h)		; Pos. Fnger
		ld	l,a			; als HL-Wert aufbereiten
		ld	h,hi(2200h)
		ld	b,8			; Geschwindigkeit
		ld	a,0ffh
taler_f1:	call	dak2
		djnz	taler_f1
		cp	04h			; '0'?
		jr	z,taler_f2
		cp	07h			; '3'?
		jr	z,taler_f3
		jr	taler_f4
; linke Grenze erreicht? wenn nicht, Punkt nach links verschieben
taler_f2:	ld	a,5
		cp	l
		jr	z,taler_f4
		res	4,(hl)
		inc	hl
		set	4,(hl)
		jr	taler_f4
; rechte Grenze erreicht? wenn nicht, Punkt nach rechts verschieben
taler_f3:	ld	a,0
		cp	l
		jr	z,taler_f4
		res	4,(hl)
		dec	hl
		set	4,(hl)
; neue Position des Fngers eintragen
taler_f4:	ld	a,l
		ld	(2220h),a
		ret
;	
		ds	1
taler_sterne:
; Prfung, ob im Anzeigebereich 1 irgendwo Bit 7 (unterer Balken) gesetzt ist
		ld	b,7
		ld	hl,2200h
taler_s1:	bit	7,(hl)
		call	nz,taler_vergl
		res	7,(hl)
		inc	hl
		djnz	taler_s1
; Prfung, ob im Anzeigebereich Bit 3 (mittlerer Balken) gesetzt ist, 
; diesen lschen, Bit 7 setzen (Verschiebung nach unten)
		ld	b,7
		ld	hl,2200h
taler_s2:	bit	3,(hl)
		jr	z,taler_s3
		set	7,(hl)
		res	3,(hl)
taler_s3:	inc	hl
		djnz	taler_s2
; Prfung, auf Bit 2 (oberer Balken), siesen lschen, Bit 3 setzen 
; (Verschiebung nach unten)
		ld	b,7
		ld	hl,2200h
taler_s4:	bit	2,(hl)
		jr	z,taler_s5
		set	3,(hl)
		res	2,(hl)
taler_s5:	inc	hl
		djnz	taler_s4
; an zuflliger Stelle neues Bit 2 (oberer Balken) setzen
		ld	a,r
		and	07h
		ld	l,a
		ld	h,hi(2200h)
		set	2,(hl)
		ret
;
taler_vergl:
		ld	a,(2220h)		; Position Fnger
		cp	l
		jr	nz,taler_v2
; Punktezhler (Einer, Zehner) um 1 erhhen
		ld	a,(2208h)
		add	a,1
		daa
		ld	(2208h),a
		jr	nc,taler_v1
; wenn bertrag, Punktezhler (Hunderter, Tausender) um 1 erhhen
		ld	a,(2209h)
		add	a,1
		daa
		ld	(2209h),a
taler_v1:	push	bc
		push	hl
		push	de
		call	SOU1K			; Ton bei Treffer, im Original SOU1K+1
		pop	de
; die rechten beiden Stellen des Anzeigebreichs 2 lschen
		ld	hl,DATLED
		ld	(hl),0
		inc	hl
		ld	(hl),0
		pop	hl
		pop	bc
taler_v2:	ret
;------------------------------------------------------------------------------

		end
		